@@ -53,8 +53,6 @@ DBusMessage *__connman_error_operation_timeout(DBusMessage *msg);
 DBusMessage *__connman_error_invalid_service(DBusMessage *msg);
 DBusMessage *__connman_error_invalid_property(DBusMessage *msg);
 
-int __connman_selftest(void);
-
 #include <connman/types.h>
 
 int __connman_manager_init(gboolean compat);
@@ -79,9 +77,14 @@ struct connman_service *service;
 
 typedef void (* passphrase_cb_t) (struct connman_service *service,
 				const char *passphrase, void *user_data);
-
+typedef void (* report_error_cb_t) (struct connman_service *service,
+				gboolean retry, void *user_data);
 int __connman_agent_request_input(struct connman_service *service,
 				passphrase_cb_t callback, void *user_data);
+int __connman_agent_report_error(struct connman_service *service,
+				const char *error,
+				report_error_cb_t callback, void *user_data);
+
 
 #include <connman/log.h>
 
@@ -105,7 +108,11 @@ void __connman_task_cleanup(void);
 
 #include <connman/inet.h>
 
-enum connman_device_type __connman_inet_get_device_type(int index);
+int __connman_inet_modify_address(int cmd, int flags, int index, int family,
+				const char *address,
+				const char *peer,
+				unsigned char prefixlen,
+				const char *broadcast);
 
 #include <connman/wifi.h>
 
@@ -119,8 +126,6 @@ void __connman_rfkill_cleanup(void);
 int __connman_resolver_init(void);
 void __connman_resolver_cleanup(void);
 
-int __connman_resolver_selftest(void);
-
 #include <connman/storage.h>
 
 int __connman_storage_init(void);
@@ -174,8 +179,6 @@ void __connman_element_list(struct connman_element *element,
 
 struct connman_service *__connman_element_get_service(struct connman_element *element);
 struct connman_device *__connman_element_get_device(struct connman_element *element);
-const char *__connman_element_get_device_path(struct connman_element *element);
-const char *__connman_element_get_network_path(struct connman_element *element);
 
 struct connman_device *__connman_element_find_device(enum connman_service_type type);
 int __connman_element_request_scan(enum connman_service_type type);
@@ -184,6 +187,16 @@ int __connman_element_disable_technology(enum connman_service_type type);
 
 gboolean __connman_element_device_isfiltered(const char *devname);
 
+int __connman_detect_init(void);
+void __connman_detect_cleanup(void);
+
+void __connman_element_set_driver(struct connman_element *element);
+
+#include <connman/proxy.h>
+
+int __connman_proxy_init(void);
+void __connman_proxy_cleanup(void);
+
 #include <connman/ipconfig.h>
 
 int __connman_ipconfig_init(void);
@@ -227,12 +240,6 @@ void __connman_ipconfig_append_ipv6(struct connman_ipconfig *ipconfig,
 void __connman_ipconfig_append_ipv6config(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter);
 int __connman_ipconfig_set_config(struct connman_ipconfig *ipconfig,
-		enum connman_ipconfig_type type, DBusMessageIter *array);
-void __connman_ipconfig_append_proxy(struct connman_ipconfig *ipconfig,
-							DBusMessageIter *iter);
-void __connman_ipconfig_append_proxyconfig(struct connman_ipconfig *ipconfig,
-							DBusMessageIter *iter);
-int __connman_ipconfig_set_proxyconfig(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *array);
 void __connman_ipconfig_append_ethernet(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter);
@@ -247,6 +254,7 @@ int __connman_ipconfig_set_gateway(struct connman_ipconfig *ipconfig,
 					struct connman_element *parent);
 int __connman_ipconfig_set_address(struct connman_ipconfig *ipconfig);
 int __connman_ipconfig_clear_address(struct connman_ipconfig *ipconfig);
+unsigned char __connman_ipconfig_netmask_prefix_len(const char *netmask);
 
 int __connman_ipconfig_set_proxy_autoconfig(struct connman_ipconfig *ipconfig,
 							const char *url);
@@ -280,18 +288,14 @@ void __connman_connection_cleanup(void);
 
 gboolean __connman_connection_update_gateway(void);
 
-int __connman_udev_init(void);
-void __connman_udev_start(void);
-void __connman_udev_cleanup(void);
-char *__connman_udev_get_devtype(const char *ifname);
-void __connman_udev_rfkill(const char *sysname, connman_bool_t blocked);
-connman_bool_t __connman_udev_get_blocked(int phyindex);
-
 int __connman_wpad_init(void);
 void __connman_wpad_cleanup(void);
-void __connman_wpad_start(struct connman_service *service);
+int __connman_wpad_start(struct connman_service *service);
 void __connman_wpad_stop(struct connman_service *service);
 
+int __connman_wispr_init(void);
+void __connman_wispr_cleanup(void);
+
 #include <connman/technology.h>
 
 void __connman_technology_list(DBusMessageIter *iter, void *user_data);
@@ -313,8 +317,10 @@ void __connman_technology_add_interface(enum connman_service_type type,
 				int index, const char *name, const char *ident);
 void __connman_technology_remove_interface(enum connman_service_type type,
 				int index, const char *name, const char *ident);
-int __connman_technology_enable_tethering(void);
-int __connman_technology_disable_tethering(void);
+int __connman_technology_enable_tethering(const char *bridge);
+int __connman_technology_disable_tethering(const char *bridge);
+
+connman_bool_t __connman_technology_get_blocked(enum connman_service_type type);
 
 #include <connman/device.h>
 
@@ -346,6 +352,8 @@ int __connman_device_disable(struct connman_device *device);
 int __connman_device_disable_persistent(struct connman_device *device);
 int __connman_device_disconnect(struct connman_device *device);
 
+connman_bool_t __connman_device_scanning(struct connman_device *device);
+
 connman_bool_t __connman_device_has_driver(struct connman_device *device);
 
 void __connman_device_set_reconnect(struct connman_device *device,
@@ -369,7 +377,8 @@ int __connman_network_disconnect(struct connman_network *network);
 int __connman_network_clear_ipconfig(struct connman_network *network,
 					struct connman_ipconfig *ipconfig);
 int __connman_network_set_ipconfig(struct connman_network *network,
-					struct connman_ipconfig *ipconfig);
+				struct connman_ipconfig *ipconfig_ipv4,
+				struct connman_ipconfig *ipconfig_ipv6);
 
 connman_bool_t __connman_network_has_driver(struct connman_network *network);
 
@@ -415,6 +424,8 @@ void __connman_tethering_cleanup(void);
 connman_bool_t __connman_tethering_get_status(void);
 int __connman_tethering_set_status(connman_bool_t status);
 void __connman_tethering_update_interface(const char *interface);
+void __connman_tethering_set_enabled(void);
+void __connman_tethering_set_disabled(void);
 
 #include <connman/provider.h>
 
@@ -423,9 +434,9 @@ void __connman_provider_list(DBusMessageIter *iter, void *user_data);
 int __connman_provider_create_and_connect(DBusMessage *msg);
 const char * __connman_provider_get_ident(struct connman_provider *provider);
 int __connman_provider_indicate_state(struct connman_provider *provider,
-				     enum connman_provider_state state);
+					enum connman_provider_state state);
 int __connman_provider_indicate_error(struct connman_provider *provider,
-				     enum connman_provider_error error);
+					enum connman_provider_error error);
 int __connman_provider_connect(struct connman_provider *provider);
 int __connman_provider_disconnect(struct connman_provider *provider);
 int __connman_provider_remove(const char *path);
@@ -450,9 +461,13 @@ struct connman_service *__connman_service_create_from_provider(struct connman_pr
 void __connman_service_update_from_network(struct connman_network *network);
 void __connman_service_remove_from_network(struct connman_network *network);
 
-void __connman_service_create_ipconfig(struct connman_service *service,
+void __connman_service_create_ip4config(struct connman_service *service,
 								int index);
-struct connman_ipconfig *__connman_service_get_ipconfig(
+void __connman_service_create_ip6config(struct connman_service *service,
+								int index);
+struct connman_ipconfig *__connman_service_get_ip4config(
+				struct connman_service *service);
+struct connman_ipconfig *__connman_service_get_ip6config(
 				struct connman_service *service);
 const char *__connman_service_get_ident(struct connman_service *service);
 const char *__connman_service_get_path(struct connman_service *service);
@@ -501,7 +516,7 @@ void __connman_service_set_proxy_autoconfig(struct connman_service *service,
 void __connman_service_set_passphrase(struct connman_service *service,
 					const char* passphrase);
 
-void __connman_service_notify(struct connman_ipconfig *ipconfig,
+void __connman_service_notify(struct connman_service *service,
 			unsigned int rx_packets, unsigned int tx_packets,
 			unsigned int rx_bytes, unsigned int tx_bytes,
 			unsigned int rx_error, unsigned int tx_error,
@@ -541,6 +556,7 @@ void __connman_notifier_connect(enum connman_service_type type);
 void __connman_notifier_disconnect(enum connman_service_type type);
 void __connman_notifier_offlinemode(connman_bool_t enabled);
 void __connman_notifier_default_changed(struct connman_service *service);
+void __connman_notifier_proxy_changed(struct connman_service *service);
 
 connman_bool_t __connman_notifier_is_registered(enum connman_service_type type);
 connman_bool_t __connman_notifier_is_enabled(enum connman_service_type type);
@@ -553,6 +569,7 @@ int __connman_rtnl_init(void);
 void __connman_rtnl_start(void);
 void __connman_rtnl_cleanup(void);
 
+enum connman_device_type __connman_rtnl_get_device_type(int index);
 unsigned int __connman_rtnl_update_interval_add(unsigned int interval);
 unsigned int __connman_rtnl_update_interval_remove(unsigned int interval);
 int __connman_rtnl_request_update(void);
@@ -588,3 +605,9 @@ int  __connman_stats_update(struct connman_service *service,
 int __connman_stats_get(struct connman_service *service,
 				connman_bool_t roaming,
 				struct connman_stats_data *data);
+
+int __connman_iptables_init(void);
+void __connman_iptables_cleanup(void);
+int __connman_iptables_command(const char *format, ...)
+				__attribute__((format(printf, 1, 2)));
+int __connman_iptables_commit(const char *table_name);
