@@ -26,8 +26,6 @@
 #include <errno.h>
 #include <string.h>
 
-#include <gdbus.h>
-
 #include "connman.h"
 
 static unsigned int hidden_counter = 0;
@@ -35,7 +33,6 @@ static unsigned int hidden_counter = 0;
 struct connman_network {
 	struct connman_element element;
 	enum connman_network_type type;
-	enum connman_network_protocol protocol;
 	connman_bool_t available;
 	connman_bool_t connected;
 	connman_bool_t roaming;
@@ -47,12 +44,10 @@ struct connman_network {
 	char *name;
 	char *node;
 	char *group;
-	struct connman_ipconfig *ipconfig;
 
 	struct connman_network_driver *driver;
 	void *driver_data;
 
-	connman_bool_t registered;
 	connman_bool_t connecting;
 	connman_bool_t associating;
 
@@ -97,153 +92,12 @@ static const char *type2string(enum connman_network_type type)
 	return NULL;
 }
 
-static DBusMessage *get_properties(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_network *network = data;
-	DBusMessage *reply;
-	DBusMessageIter array, dict;
-
-	DBG("conn %p", conn);
-
-	reply = dbus_message_new_method_return(msg);
-	if (reply == NULL)
-		return NULL;
-
-	dbus_message_iter_init_append(reply, &array);
-
-	connman_dbus_dict_open(&array, &dict);
-
-	if (network->device) {
-		const char *path = connman_device_get_path(network->device);
-		if (path != NULL)
-			connman_dbus_dict_append_basic(&dict, "Device",
-						DBUS_TYPE_OBJECT_PATH, &path);
-	}
-
-	if (network->address != NULL)
-		connman_dbus_dict_append_basic(&dict, "Address",
-					DBUS_TYPE_STRING, &network->address);
-
-	if (network->name != NULL)
-		connman_dbus_dict_append_basic(&dict, "Name",
-					DBUS_TYPE_STRING, &network->name);
-
-	connman_dbus_dict_append_basic(&dict, "Connected",
-				DBUS_TYPE_BOOLEAN, &network->connected);
-
-	if (network->strength > 0)
-		connman_dbus_dict_append_basic(&dict, "Strength",
-					DBUS_TYPE_BYTE, &network->strength);
-
-	if (network->frequency > 0)
-		connman_dbus_dict_append_basic(&dict, "Frequency",
-					DBUS_TYPE_UINT16, &network->frequency);
-
-	if (network->wifi.ssid != NULL && network->wifi.ssid_len > 0)
-		connman_dbus_dict_append_fixed_array(&dict, "WiFi.SSID",
-				DBUS_TYPE_BYTE, &network->wifi.ssid,
-						network->wifi.ssid_len);
-
-	if (network->wifi.mode != NULL)
-		connman_dbus_dict_append_basic(&dict, "WiFi.Mode",
-				DBUS_TYPE_STRING, &network->wifi.mode);
-
-	if (network->wifi.channel > 0)
-		connman_dbus_dict_append_basic(&dict, "WiFi.Channel",
-				DBUS_TYPE_UINT16, &network->wifi.channel);
-
-	if (network->wifi.security != NULL) {
-		connman_dbus_dict_append_basic(&dict, "WiFi.Security",
-				DBUS_TYPE_STRING, &network->wifi.security);
-
-		if (g_strcmp0(network->wifi.security, "ieee8021x") == 0 &&
-						    network->wifi.eap != NULL)
-			connman_dbus_dict_append_basic(&dict, "WiFi.EAP",
-					DBUS_TYPE_STRING, &network->wifi.eap);
-	}
-
-
-	if (network->wifi.passphrase != NULL)
-		connman_dbus_dict_append_basic(&dict, "WiFi.Passphrase",
-				DBUS_TYPE_STRING, &network->wifi.passphrase);
-
-	connman_dbus_dict_close(&array, &dict);
-
-	return reply;
-}
-
-static GDBusMethodTable network_methods[] = {
-	{ "GetProperties", "",   "a{sv}", get_properties },
-	{ },
-};
-
-static GDBusSignalTable network_signals[] = {
-	{ "PropertyChanged", "sv" },
-	{ },
-};
-
-static DBusConnection *connection;
-
-static void append_networks(DBusMessageIter *iter, void *user_data)
-{
-	struct connman_device *device = user_data;
-
-	__connman_element_list((struct connman_element *) device,
-					CONNMAN_ELEMENT_TYPE_NETWORK, iter);
-}
-
-static void emit_networks_signal(struct connman_device *device)
-{
-	const char *path = connman_device_get_path(device);
-
-	connman_dbus_property_changed_array(path,
-			CONNMAN_DEVICE_INTERFACE, "Networks",
-			DBUS_TYPE_OBJECT_PATH, append_networks, device);
-}
-
-static int register_interface(struct connman_element *element)
-{
-	struct connman_network *network = element->network;
-
-	DBG("element %p name %s", element, element->name);
-
-	if (g_dbus_register_interface(connection, element->path,
-					CONNMAN_NETWORK_INTERFACE,
-					network_methods, network_signals,
-					NULL, network, NULL) == FALSE) {
-		connman_error("Failed to register %s network", element->path);
-		return -EIO;
-	}
-
-	network->registered = TRUE;
-
-	emit_networks_signal(network->device);
-
-	return 0;
-}
-
-static void unregister_interface(struct connman_element *element)
-{
-	struct connman_network * network = element->network;
-
-	DBG("element %p name %s", element, element->name);
-
-	network->registered = FALSE;
-
-	if (network->device != NULL)
-		emit_networks_signal(network->device);
-
-	g_dbus_unregister_interface(connection, element->path,
-						CONNMAN_NETWORK_INTERFACE);
-}
-
 connman_bool_t __connman_network_has_driver(struct connman_network *network)
 {
 	if (network == NULL || network->driver == NULL)
 		return FALSE;
 
-	return network->registered;
+	return TRUE;
 }
 
 static GSList *driver_list = NULL;
@@ -266,8 +120,18 @@ static gint compare_priority(gconstpointer a, gconstpointer b)
  */
 int connman_network_driver_register(struct connman_network_driver *driver)
 {
+	GSList *list;
+
 	DBG("driver %p name %s", driver, driver->name);
 
+	for (list = driver_list; list; list = list->next) {
+		struct connman_network_driver *tmp = list->data;
+
+		if (tmp->type == driver->type)
+			return -EALREADY;
+
+	}
+
 	driver_list = g_slist_insert_sorted(driver_list, driver,
 							compare_priority);
 
@@ -297,6 +161,13 @@ static void network_destruct(struct connman_element *element)
 	g_free(network->wifi.mode);
 	g_free(network->wifi.security);
 	g_free(network->wifi.passphrase);
+	g_free(network->wifi.eap);
+	g_free(network->wifi.identity);
+	g_free(network->wifi.ca_cert_path);
+	g_free(network->wifi.client_cert_path);
+	g_free(network->wifi.private_key_path);
+	g_free(network->wifi.private_key_passphrase);
+	g_free(network->wifi.phase2_auth);
 
 	g_free(network->group);
 	g_free(network->node);
@@ -304,11 +175,6 @@ static void network_destruct(struct connman_element *element)
 	g_free(network->address);
 	g_free(network->identifier);
 
-	if (network->ipconfig) {
-		connman_ipconfig_unref(network->ipconfig);
-		network->ipconfig = NULL;
-	}
-
 	network->device = NULL;
 }
 
@@ -338,7 +204,6 @@ struct connman_network *connman_network_create(const char *identifier,
 
 	__connman_element_initialize(&network->element);
 
-	//temp = connman_dbus_encode_string(identifier);
 	if (identifier == NULL) {
 		temp = g_strdup_printf("hidden_%d", hidden_counter++);
 		network->hidden = TRUE;
@@ -420,17 +285,6 @@ const char *connman_network_get_identifier(struct connman_network *network)
 	return network->identifier;
 }
 
-/**
- * connman_network_get_path:
- * @network: network structure
- *
- * Get path name of network
- */
-const char *connman_network_get_path(struct connman_network *network)
-{
-	return network->element.path;
-}
-
 /**
  * connman_network_set_index:
  * @network: network structure
@@ -447,16 +301,25 @@ void connman_network_set_index(struct connman_network *network, int index)
 	if (service == NULL)
 		goto done;
 
-	ipconfig = __connman_service_get_ipconfig(service);
+	ipconfig = __connman_service_get_ip4config(service);
 
-	if (network->element.index < 0 && ipconfig == NULL)
-		/*
-		 * This is needed for plugins that havent set their ipconfig
-		 * layer yet, due to not being able to get a network index
-		 * prior to creating a service.
-		 */
-		__connman_service_create_ipconfig(service, index);
-	else {
+	DBG("index %d service %p ip4config %p", network->element.index,
+		service, ipconfig);
+
+	if (network->element.index < 0 && ipconfig == NULL) {
+
+		ipconfig = __connman_service_get_ip4config(service);
+		if (ipconfig == NULL)
+			/*
+			 * This is needed for plugins that havent set their
+			 * ipconfig layer yet, due to not being able to get
+			 * a network index prior to creating a service.
+			 */
+			__connman_service_create_ip4config(service, index);
+		else
+			__connman_ipconfig_set_index(ipconfig, index);
+
+	} else {
 		/* If index changed, the index of ipconfig must be reset. */
 		if (ipconfig == NULL)
 			goto done;
@@ -491,19 +354,6 @@ struct connman_element *connman_network_get_element(
 	return &network->element;
 }
 
-/**
- * connman_network_set_protocol:
- * @network: network structure
- * @protocol: network protocol
- *
- * Change protocol of network
- */
-void connman_network_set_protocol(struct connman_network *network,
-					enum connman_network_protocol protocol)
-{
-	network->protocol = protocol;
-}
-
 /**
  * connman_network_set_group:
  * @network: network structure
@@ -528,24 +378,21 @@ void connman_network_set_group(struct connman_network *network,
 	}
 
 	if (g_strcmp0(network->group, group) == 0) {
-		if (group != NULL && network->registered)
+		if (group != NULL)
 			__connman_profile_update_network(network);
 		return;
 	}
 
 	if (network->group != NULL) {
-		if (network->registered)
-			__connman_profile_remove_network(network);
+		__connman_profile_remove_network(network);
 
 		g_free(network->group);
 	}
 
 	network->group = g_strdup(group);
 
-	if (network->group != NULL) {
-		if (network->registered)
+	if (network->group != NULL)
 			__connman_profile_add_network(network);
-	}
 }
 
 /**
@@ -683,7 +530,7 @@ static void set_configure_error(struct connman_network *network)
 					CONNMAN_SERVICE_STATE_FAILURE);
 }
 
-void connman_network_set_method(struct connman_network *network,
+void connman_network_set_ipv4_method(struct connman_network *network,
 					enum connman_ipconfig_method method)
 {
 	struct connman_service *service;
@@ -695,7 +542,26 @@ void connman_network_set_method(struct connman_network *network,
 	if (service == NULL)
 		return;
 
-	ipconfig = __connman_service_get_ipconfig(service);
+	ipconfig = __connman_service_get_ip4config(service);
+	if (ipconfig == NULL)
+		return;
+
+	connman_ipconfig_set_method(ipconfig, method);
+}
+
+void connman_network_set_ipv6_method(struct connman_network *network,
+					enum connman_ipconfig_method method)
+{
+	struct connman_service *service;
+	struct connman_ipconfig *ipconfig;
+
+	network->element.ipv6.method = method;
+
+	service = __connman_service_lookup_from_network(network);
+	if (service == NULL)
+		return;
+
+	ipconfig = __connman_service_get_ip6config(service);
 	if (ipconfig == NULL)
 		return;
 
@@ -789,7 +655,7 @@ static void set_connected_manual(struct connman_network *network)
 
 	service = __connman_service_lookup_from_network(network);
 
-	ipconfig = __connman_service_get_ipconfig(service);
+	ipconfig = __connman_service_get_ip4config(service);
 
 	set_configuration(network);
 
@@ -821,9 +687,6 @@ static int set_connected_dhcp(struct connman_network *network)
 
 	DBG("network %p", network);
 
-	if (network->protocol != CONNMAN_NETWORK_PROTOCOL_IP)
-		return -EINVAL;
-
 	element = connman_element_create(NULL);
 	if (element == NULL)
 		return -ENOMEM;
@@ -871,31 +734,34 @@ static gboolean set_connected(gpointer user_data)
 {
 	struct connman_network *network = user_data;
 	struct connman_service *service;
-	struct connman_ipconfig *ipconfig;
-	enum connman_ipconfig_method method;
+	struct connman_ipconfig *ipconfig_ipv4, *ipconfig_ipv6;
+	enum connman_ipconfig_method ipv4_method, ipv6_method;
 
 	service = __connman_service_lookup_from_network(network);
 
-	ipconfig = __connman_service_get_ipconfig(service);
+	ipconfig_ipv4 = __connman_service_get_ip4config(service);
+	ipconfig_ipv6 = __connman_service_get_ip6config(service);
 
-	method = __connman_ipconfig_get_method(ipconfig);
+	DBG("service %p ipv4 %p ipv6 %p", service, ipconfig_ipv4,
+		ipconfig_ipv6);
 
-	DBG("method %d", method);
+	ipv4_method = __connman_ipconfig_get_method(ipconfig_ipv4);
+	ipv6_method = __connman_ipconfig_get_method(ipconfig_ipv6);
+
+	DBG("method ipv4 %d ipv6 %d", ipv4_method, ipv6_method);
+	DBG("network connected %d", network->connected);
 
 	if (network->connected == TRUE) {
-		enum connman_ipconfig_method ipv6_method;
-		struct connman_ipconfig *ipv6config;
 		int ret;
 
-		ipv6config = connman_ipconfig_get_ipv6config(ipconfig);
-		ipv6_method = __connman_ipconfig_get_method(ipv6config);
 		switch (ipv6_method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_AUTO:
 			break;
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
 		case CONNMAN_IPCONFIG_METHOD_MANUAL:
-			ret = manual_ipv6_set(network, ipv6config);
+			ret = manual_ipv6_set(network, ipconfig_ipv6);
 			if (ret != 0) {
 				connman_network_set_error(network,
 					CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
@@ -906,9 +772,10 @@ static gboolean set_connected(gpointer user_data)
 			break;
 		}
 
-		switch (method) {
+		switch (ipv4_method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_AUTO:
 			return FALSE;
 		case CONNMAN_IPCONFIG_METHOD_FIXED:
 			if (set_connected_fixed(network) < 0) {
@@ -936,8 +803,6 @@ static gboolean set_connected(gpointer user_data)
 		__connman_device_set_network(network->device, NULL);
 		network->hidden = FALSE;
 
-		__connman_device_decrease_connections(network->device);
-
 		service = __connman_service_lookup_from_network(network);
 
 		__connman_service_indicate_state(service,
@@ -973,17 +838,11 @@ int connman_network_set_connected(struct connman_network *network,
 	if (network->connected == connected)
 		return -EALREADY;
 
+	if (connected == FALSE)
+		__connman_device_decrease_connections(network->device);
+
 	network->connected = connected;
 
-	if (network->registered == FALSE) {
-		g_idle_add(set_connected, network);
-		return 0;
-	}
-
-	connman_dbus_property_changed_basic(network->element.path,
-				CONNMAN_NETWORK_INTERFACE, "Connected",
-						DBUS_TYPE_BOOLEAN, &connected);
-
 	set_connected(network);
 
 	return 0;
@@ -1039,10 +898,10 @@ int __connman_network_connect(struct connman_network *network)
 	if (network->device == NULL)
 		return -ENODEV;
 
-	__connman_device_disconnect(network->device);
-
 	network->connecting = TRUE;
 
+	__connman_device_disconnect(network->device);
+
 	service = __connman_service_lookup_from_network(network);
 
 	err = network->driver->connect(network);
@@ -1101,9 +960,6 @@ static int dhcp_start(struct connman_network *network)
 	struct connman_element *element;
 	int error;
 
-	if (network->protocol != CONNMAN_NETWORK_PROTOCOL_IP)
-		return -EINVAL;
-
 	element = connman_element_create(NULL);
 	if (element == NULL)
 		return -ENOMEM;
@@ -1122,9 +978,6 @@ static int dhcp_start(struct connman_network *network)
 
 static int dhcp_stop(struct connman_network *network)
 {
-	if (network->protocol != CONNMAN_NETWORK_PROTOCOL_IP)
-		return -EINVAL;
-
 	connman_element_unregister_children_type(&network->element,
 					CONNMAN_ELEMENT_TYPE_CONNECTION);
 	connman_element_unregister_children_type(&network->element,
@@ -1175,6 +1028,7 @@ int __connman_network_clear_ipconfig(struct connman_network *network,
 	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
 	case CONNMAN_IPCONFIG_METHOD_OFF:
 	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_AUTO:
 		return -EINVAL;
 	case CONNMAN_IPCONFIG_METHOD_MANUAL:
 		connman_element_unregister_children_type(&network->element,
@@ -1193,42 +1047,48 @@ int __connman_network_clear_ipconfig(struct connman_network *network,
 }
 
 int __connman_network_set_ipconfig(struct connman_network *network,
-					struct connman_ipconfig *ipconfig)
+					struct connman_ipconfig *ipconfig_ipv4,
+					struct connman_ipconfig *ipconfig_ipv6)
 {
-	struct connman_ipconfig *ipv6config;
 	enum connman_ipconfig_method method;
 	int ret;
 
-	ipv6config = connman_ipconfig_get_ipv6config(ipconfig);
-	method = __connman_ipconfig_get_method(ipv6config);
-	switch (method) {
-	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
-	case CONNMAN_IPCONFIG_METHOD_OFF:
-		break;
-	case CONNMAN_IPCONFIG_METHOD_FIXED:
-	case CONNMAN_IPCONFIG_METHOD_MANUAL:
-		ret = manual_ipv6_set(network, ipv6config);
-		if (ret != 0) {
-			connman_network_set_error(network,
-				CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
-			return FALSE;
+	if (ipconfig_ipv6) {
+		method = __connman_ipconfig_get_method(ipconfig_ipv6);
+
+		switch (method) {
+		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_AUTO:
+			break;
+		case CONNMAN_IPCONFIG_METHOD_FIXED:
+		case CONNMAN_IPCONFIG_METHOD_MANUAL:
+			ret = manual_ipv6_set(network, ipconfig_ipv6);
+			if (ret != 0) {
+				connman_network_set_error(network,
+					CONNMAN_NETWORK_ERROR_ASSOCIATE_FAIL);
+				return FALSE;
+			}
+			break;
+		case CONNMAN_IPCONFIG_METHOD_DHCP:
+			break;
 		}
-		break;
-	case CONNMAN_IPCONFIG_METHOD_DHCP:
-		break;
 	}
 
-	method = __connman_ipconfig_get_method(ipconfig);
+	if (ipconfig_ipv4) {
+		method = __connman_ipconfig_get_method(ipconfig_ipv4);
 
-	switch (method) {
-	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
-	case CONNMAN_IPCONFIG_METHOD_OFF:
-	case CONNMAN_IPCONFIG_METHOD_FIXED:
-		return -EINVAL;
-	case CONNMAN_IPCONFIG_METHOD_MANUAL:
-		return manual_ipv4_set(network, ipconfig);
-	case CONNMAN_IPCONFIG_METHOD_DHCP:
-		return dhcp_start(network);
+		switch (method) {
+		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_FIXED:
+		case CONNMAN_IPCONFIG_METHOD_AUTO:
+			return -EINVAL;
+		case CONNMAN_IPCONFIG_METHOD_MANUAL:
+			return manual_ipv4_set(network, ipconfig_ipv4);
+		case CONNMAN_IPCONFIG_METHOD_DHCP:
+			return dhcp_start(network);
+		}
 	}
 
 	return 0;
@@ -1673,7 +1533,6 @@ static int network_probe(struct connman_element *element)
 {
 	struct connman_network *network = element->network;
 	GSList *list;
-	int err;
 
 	DBG("element %p name %s", element, element->name);
 
@@ -1697,13 +1556,6 @@ static int network_probe(struct connman_element *element)
 	if (network->driver == NULL)
 		return -ENODEV;
 
-	err = register_interface(element);
-	if (err < 0) {
-		if (network->driver->remove)
-			network->driver->remove(network);
-		return err;
-	}
-
 	switch (network->type) {
 	case CONNMAN_NETWORK_TYPE_UNKNOWN:
 	case CONNMAN_NETWORK_TYPE_VENDOR:
@@ -1745,8 +1597,7 @@ static void network_remove(struct connman_element *element)
 	case CONNMAN_NETWORK_TYPE_WIFI:
 	case CONNMAN_NETWORK_TYPE_WIMAX:
 		if (network->group != NULL) {
-			if (network->registered)
-				__connman_profile_remove_network(network);
+			__connman_profile_remove_network(network);
 
 			g_free(network->group);
 			network->group = NULL;
@@ -1754,8 +1605,6 @@ static void network_remove(struct connman_element *element)
 		break;
 	}
 
-	unregister_interface(element);
-
 	if (network->driver->remove)
 		network->driver->remove(network);
 }
@@ -1800,8 +1649,6 @@ int __connman_network_init(void)
 {
 	DBG("");
 
-	connection = connman_dbus_get_connection();
-
 	return connman_driver_register(&network_driver);
 }
 
@@ -1810,6 +1657,4 @@ void __connman_network_cleanup(void)
 	DBG("");
 
 	connman_driver_unregister(&network_driver);
-
-	dbus_connection_unref(connection);
 }
